﻿using AccountContracts.BindingModels;
using AccountContracts.SearchModels;
using AccountContracts.StoragesContracts;
using AccountContracts.ViewModels;
using AccountDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AccountDatabaseImplement.Implements
{
    public class AccountStorage : IAccountStorage
    {
        public List<AccountViewModel> GetFullList()
        {
            using var context = new AccountDatabase();
            return context.Accounts.Include(x => x.Role)
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public AccountViewModel? GetElement(AccountSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }

            using var context = new AccountDatabase();
            return context.Accounts.Include(x => x.Role)
            .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
            ?.GetViewModel;
        }
        public AccountViewModel? Insert(AccountBindingModel model)
        {
            using var context = new AccountDatabase();
            var newAccount = Account.Create(model, context);
            if (newAccount == null)
            {
                return null;
            }
            context.Accounts.Add(newAccount);
            context.SaveChanges();
            return newAccount.GetViewModel;
        }
        public AccountViewModel? Update(AccountBindingModel model)
        {
            using var context = new AccountDatabase();
            var account = context.Accounts.FirstOrDefault(x => x.Id == model.Id);
            if (account == null)
            {
                return null;
            }
            account.Update(model);
            context.SaveChanges();
            return account.GetViewModel;
        }
        public AccountViewModel? Delete(AccountBindingModel model)
        {
            using var context = new AccountDatabase();
            var element = context.Accounts.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Accounts.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
