﻿using Microsoft.Office.Interop.Excel;
using Microsoft.Office.Interop.Word;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace NevaevaLibrary.LogicalComponents
{
    public partial class WordDiagramComponent : Component
    {
        public WordDiagramComponent()
        {
            InitializeComponent();
        }

        public WordDiagramComponent(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }

        private object missing = System.Reflection.Missing.Value;

        private string GetExcelColumnName(int columnNumber)
        {
            string columnName = "";

            while (columnNumber > 0)
            {
                int modulo = (columnNumber - 1) % 26;
                columnName = Convert.ToChar('A' + modulo) + columnName;
                columnNumber = (columnNumber - modulo) / 26;
            }

            return columnName;
        }

        public void createWithDiagram<T>(string path, string title, string diagramTitle, DiagramLegendEnum diagramLegendAnchor, List<T> data, string seriesNameField, string valueField)
        {
            var excelApp = new Microsoft.Office.Interop.Excel.Application { SheetsInNewWorkbook = 1 };
            Workbook workbook = excelApp.Workbooks.Add(Type.Missing);
            Worksheet worksheet = (Worksheet)workbook.Worksheets.get_Item(1);

            FieldInfo? seriesName = typeof(T).GetField(seriesNameField);
            FieldInfo? value = typeof(T).GetField(valueField);
            if (seriesName == null || value == null) throw new ArgumentException("Переданного поля не существует");
            int offsetX = 1;
            int offsetYMax = -1;
            foreach (var item in data)
            {
                string columnChar = GetExcelColumnName(offsetX);
                var cell = worksheet.get_Range(columnChar + "2", columnChar + "2");
                cell.Font.Size = 14;
                cell.Font.Name = "Times New Roman";
                cell.ColumnWidth = 8;
                cell.RowHeight = 25;
                cell.HorizontalAlignment = Constants.xlCenter;
                cell.VerticalAlignment = Constants.xlCenter;
                cell.Value2 = seriesName.GetValue(item);

                int offsetY = 3;

                foreach (var val in (value.GetValue(item) as IEnumerable))
                {
                    cell = worksheet.get_Range(columnChar + offsetY, columnChar + offsetY);
                    cell.Value2 = val;

                    offsetY++;
                }
                if (offsetY > offsetYMax) offsetYMax = offsetY;

                offsetX++;
            }

            var charts = worksheet.ChartObjects() as ChartObjects;
            int chartWidth = 300;
            int chartHeight = 300;
            var chartObject = charts.Add(250, 10, chartWidth, chartHeight);
            var chart = chartObject.Chart;
            var endColumn = GetExcelColumnName(offsetX - 1);
            var range = worksheet.get_Range($"A2", endColumn + (offsetYMax - 1));
            chart.SetSourceData(range);
            chart.ChartType = XlChartType.xlLine;
            switch (diagramLegendAnchor)
            {
                case DiagramLegendEnum.TopLeft:
                    chart.Legend.Top = 0;
                    chart.Legend.Left = 0;
                    break;
                case DiagramLegendEnum.TopRight:
                    chart.Legend.Top = 0;
                    chart.Legend.Left = chartWidth - chart.Legend.Width;
                    break;
                case DiagramLegendEnum.BottomLeft:
                    chart.Legend.Top = chartHeight - chart.Legend.Height;
                    chart.Legend.Left = 0;
                    break;
                case DiagramLegendEnum.BottomRight:
                    chart.Legend.Top = chartHeight - chart.Legend.Height;
                    chart.Legend.Left = chartWidth - chart.Legend.Width;
                    break;
            }
            chart.ChartWizard(Source: range, Title: diagramTitle);

            chart.Export(path.Substring(0, path.Length - 5) + ".jpg", "JPG", false);

            workbook.Close(SaveChanges: false);
            excelApp.Quit();

            var winword = new Microsoft.Office.Interop.Word.Application();
            var document = winword.Documents.Add(ref missing, ref missing, ref missing, ref missing);
            document.PageSetup.RightMargin = 50;
            document.PageSetup.LeftMargin = 50;

            var header = document.Content.Paragraphs.Add(Type.Missing);
            header.Range.Text = title;
            header.Format.Alignment = WdParagraphAlignment.wdAlignParagraphCenter;
            header.Range.Font.Name = "Times New Roman";
            header.Range.Font.Size = 22;
            header.Range.Font.Bold = 2;
            header.Format.SpaceAfter = 18;
            header.Range.InsertParagraphAfter();

            document.Shapes.AddPicture(path.Substring(0, path.Length - 5) + ".jpg", Top: 200);

            document.SaveAs(path, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing);
            document.Close(ref missing, ref missing, ref missing);
            document = null;
            winword.Quit(ref missing, ref missing, ref missing);
        }
    }
}
