﻿using Microsoft.Office.Interop.Word;
using Microsoft.VisualBasic;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace NevaevaLibrary.LogicalComponents
{
    public partial class WordTableComponent : Component
    {
        public WordTableComponent()
        {
            InitializeComponent();
        }

        public WordTableComponent(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }

        private object missing = System.Reflection.Missing.Value;

        public void createWithTable<T>(string path, string title, List<(int, int)> merges, List<int> widths, List<(string, string)> headers, List<T> items)
        {
            if (merges.Count == 0 || widths.Count == 0 || headers.Count == 0 || items.Count == 0) throw new ArgumentException("Недостаточно данных");
            int[] cellsArray = new int[widths.Count];
            foreach (var merge in merges)
            {
                if (merge.Item1 >= merge.Item2) throw new ArgumentException("Неправильно заполнены объединения строк");
                for (int i = merge.Item1; i < merge.Item2 + 1; i++)
                {
                    cellsArray[i]++;
                }
            }
            foreach (int cell in cellsArray)
            {
                if (cell > 1) throw new ArgumentException("Объединения заходят друг на друга");
            }

            var winword = new Microsoft.Office.Interop.Word.Application();
            var document = winword.Documents.Add(ref missing, ref missing, ref missing, ref missing);
            document.PageSetup.RightMargin = 50;
            document.PageSetup.LeftMargin = 50;
            document.PageSetup.Orientation = WdOrientation.wdOrientLandscape;

            var header = document.Content.Paragraphs.Add(Type.Missing);
            header.Range.Text = title;
            header.Format.Alignment = WdParagraphAlignment.wdAlignParagraphCenter;
            header.Range.Font.Name = "Times New Roman";
            header.Range.Font.Size = 22;
            header.Range.Font.Bold = 2;
            header.Format.SpaceAfter = 18;
            header.Range.InsertParagraphAfter();

            var table = document.Tables.Add(document.Bookmarks.get_Item("\\endofdoc").Range,
                 items.Count + 2, widths.Count, Type.Missing, Type.Missing);
            table.Borders.Enable = 1;

            for (int i = 0; i < widths.Count; i++)
            {
                table.Cell(1, i + 1).Width = widths[i];
                table.Cell(1, i + 1).Height = 20;
                table.Cell(2, i + 1).Width = widths[i];
                table.Cell(2, i + 1).Height = 20;
            }

            //checks
            List<Cell> ranges = new List<Cell>();
            foreach (var merge in merges)
            {
                ranges.Add(table.Rows[1].Cells[merge.Item1 + 1]);
            }

            //number of merged cell
            int rangeIndex = 0;
            //number of cell
            int headerIndex = 0;
            List<PropertyInfo> cellProperties = new List<PropertyInfo>();
            var type = typeof(T);
            for (int i = 0; i < widths.Count; i++)
            {
                if (cellsArray[i] == 1)
                {
                    //work with merge
                    if (!string.IsNullOrEmpty(headers[headerIndex].Item1)) throw new ArgumentException("Заголовки и объединения строк не совпадают");

                    var headerCell = ranges[rangeIndex];
                    headerCell.Range.Text = headers[headerIndex].Item2;
                    headerCell.Range.Font.Size = 11;
                    headerIndex++;

                    //work with cells in merge
                    for (; i <= merges[rangeIndex].Item2; i++)
                    {
                        //work with cell
                        if (string.IsNullOrEmpty(headers[headerIndex].Item1)) throw new ArgumentException("Заголовки и объединения строк не совпадают");
                        var property = type.GetProperty(headers[headerIndex].Item1);
                        if (property == null) throw new ArgumentException("В заголовках указано поле, которого нет в переданном классе");
                        //format header
                        var cell = table.Cell(2, i + 1);
                        cell.Range.Text = headers[headerIndex].Item2;
                        cell.Width = widths[i];
                        cell.Range.Font.Size = 11;

                        cellProperties.Add(property);
                        headerIndex++;
                    }
                    i--;
                    rangeIndex++;
                }
                else
                {
                    //work with cell
                    if (string.IsNullOrEmpty(headers[headerIndex].Item1)) throw new ArgumentException("Заголовки и объединения строк не совпадают");
                    var property = type.GetProperty(headers[headerIndex].Item1);
                    if (property == null) throw new ArgumentException("В заголовках указано поле, которого нет в переданном классе");
                    //format header
                    var cell = table.Cell(1, i + 1);

                    cell.Merge(table.Cell(2, i + 1));
                    cell.Range.Text = headers[headerIndex].Item2;
                    cell.Width = widths[i];
                    cell.Range.Font.Size = 11;

                    cellProperties.Add(property);
                    headerIndex++;
                }
            }

            int xOffset = 0;
            foreach (var merge in merges)
            {
                table.Cell(1, merge.Item1 + 1 - xOffset).Merge(table.Cell(1, merge.Item2 + 1 - xOffset));
                xOffset += merge.Item2 - merge.Item1;
            }

            int rowNum = 3;
            foreach (T item in items)
            {
                int columnNum = 1;
                foreach (var cellProperty in cellProperties)
                {
                    var cell = table.Cell(rowNum, columnNum);
                    cell.Range.Text = cellProperty.GetValue(item)?.ToString();
                    cell.Width = widths[columnNum - 1];
                    cell.Range.Bold = 0;
                    cell.Range.Font.Size = 11;
                    cell.Height = 20;

                    columnNum++;
                }
                rowNum++;
            }

            document.SaveAs(path, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing);
            document.Close(ref missing, ref missing, ref missing);
            document = null;
            winword.Quit(ref missing, ref missing, ref missing);
        }
    }
}
